@@ -1,5 +1,7 @@
+import json
 import logging
 import tempfile
+from typing import Any
 
 _, LOG_FILE = tempfile.mkstemp(suffix=".log")
 
@@ -30,3 +32,13 @@ debug_files = []
 
 def add_debug_file(file: str) -> None:
     debug_files.append(file)
+
+
+def json_dump_to_debug_file(value: Any, file_name: str) -> str:
+
+    out_file = f"{tempfile.mkdtemp()}/{file_name}"
+    with open(out_file, "w") as fp:
+        fp.write(json.dumps(value, default=str))
+
+    add_debug_file(out_file)
+    return out_file
