@@ -1,3 +1,5 @@
+from typing import Optional
+
 from pydantic.dataclasses import dataclass
 
 from metaphor.common.base_config import BaseConfig
@@ -5,14 +7,17 @@ from metaphor.common.base_config import BaseConfig
 
 @dataclass
 class MssqlConfig(BaseConfig):
-    # Azure Directory (tenant) ID
-    tenant_id: str
-
-    # SQL Server name
-    server_name: str
-
     # The database username
     username: str
 
     # The database password
     password: str
+
+    # SQL Server endpoint. Instead of Azure SQL endpoint if specified
+    endpoint: str
+
+    # Azure Directory (tenant) ID
+    tenant_id: Optional[str] = ""
+
+    # SQL Server name
+    server_name: Optional[str] = ""
