@@ -2,7 +2,7 @@ from typing import Optional
 
 from pydantic.dataclasses import dataclass
 
-from metaphor.mssql.config import MssqlConfig
+from metaphor.common.base_config import BaseConfig
 
 
 @dataclass
@@ -12,5 +12,17 @@ class SynapseQueryLogConfig:
 
 
 @dataclass
-class SynapseConfig(MssqlConfig):
+class SynapseConfig(BaseConfig):
+    # Azure Directory (tenant) ID
+    tenant_id: str
+
+    # SQL Server name
+    server_name: str
+
+    # The database username
+    username: str
+
+    # The database password
+    password: str
+
     query_log: Optional[SynapseQueryLogConfig] = SynapseQueryLogConfig()
